@@ -20,6 +20,7 @@
 
 '''Administration commands for maintaining Roundup trackers.
 '''
+__docformat__ = 'restructuredtext'
 
 import sys, os, getpass, getopt, re, UserDict, shutil, rfc822
 from roundup import date, hyperdb, roundupdb, init, password, token, rcsv
@@ -277,11 +278,18 @@ Command help:
     def listTemplates(self):
         ''' List all the available templates.
 
-            Look in three places:
-                <prefix>/share/roundup/templates/*
-                <__file__>/../templates/*
-                current dir/*
-                current dir as a template
+        Look in the following places, where the later rules take precedence:
+
+         1. <prefix>/share/roundup/templates/*
+            this should be the standard place to find them when Roundup is
+            installed
+         2. <roundup.admin.__file__>/../templates/*
+            this will be used if Roundup's run in the distro (aka. source)
+            directory
+         3. <current working dir>/*
+            this is for when someone unpacks a 3rd-party template
+         4. <current working dir>
+            this is for someone who "cd"s to the 3rd-party template dir
         '''
         # OK, try <prefix>/share/roundup/templates
         # -- this module (roundup.admin) will be installed in something
@@ -348,11 +356,23 @@ Command help:
             raise UsageError, _('Not enough arguments supplied')
 
         # make sure the tracker home can be created
+        tracker_home = os.path.abspath(tracker_home)
         parent = os.path.split(tracker_home)[0]
         if not os.path.exists(parent):
             raise UsageError, _('Instance home parent directory "%(parent)s"'
                 ' does not exist')%locals()
 
+        if os.path.exists(os.path.join(tracker_home, 'config.py')):
+            print _('WARNING: There appears to be a tracker in '
+                '"%(tracker_home)s"!')%locals()
+            print _('If you re-install it, you will lose all the data!')
+            ok = raw_input(_('Erase it? Y/N: ')).strip()
+            if ok.strip().lower() != 'y':
+                return 0
+
+            # clear it out so the install isn't confused
+            shutil.rmtree(tracker_home)
+
         # select template
         templates = self.listTemplates()
         template = len(args) > 1 and args[1] or ''
@@ -431,8 +451,8 @@ Command help:
         if db_exists:
             print _('WARNING: The database is already initialised!')
             print _('If you re-initialise it, you will lose all the data!')
-            ok = raw_input(_('Erase it? Y/[N]: ')).strip()
-            if ok.lower() != 'y':
+            ok = raw_input(_('Erase it? Y/N: ')).strip()
+            if ok.strip().lower() != 'y':
                 return 0
 
             # Get a database backend in use by tracker
@@ -562,42 +582,11 @@ Command help:
 
             properties = cl.getprops()
             for key, value in props.items():
-                proptype =  properties[key]
-                if isinstance(proptype, hyperdb.Multilink):
-                    if value is None:
-                        props[key] = []
-                    else:
-                        props[key] = value.split(',')
-                elif value is None:
-                    continue
-                elif isinstance(proptype, hyperdb.String):
-                    continue
-                elif isinstance(proptype, hyperdb.Password):
-                    m = pwre.match(value)
-                    if m:
-                        # password is being given to us encrypted
-                        p = password.Password()
-                        p.scheme = m.group(1)
-                        p.password = m.group(2)
-                        props[key] = p
-                    else:
-                        props[key] = password.Password(value)
-                elif isinstance(proptype, hyperdb.Date):
-                    try:
-                        props[key] = date.Date(value)
-                    except ValueError, message:
-                        raise UsageError, '"%s": %s'%(value, message)
-                elif isinstance(proptype, hyperdb.Interval):
-                    try:
-                        props[key] = date.Interval(value)
-                    except ValueError, message:
-                        raise UsageError, '"%s": %s'%(value, message)
-                elif isinstance(proptype, hyperdb.Link):
-                    props[key] = value
-                elif isinstance(proptype, hyperdb.Boolean):
-                    props[key] = value.lower() in ('yes', 'true', 'on', '1')
-                elif isinstance(proptype, hyperdb.Number):
-                    props[key] = float(value)
+                try:
+                    props[key] = hyperdb.rawToHyperdb(self.db, cl, itemid,
+                        key, value)
+                except hyperdb.HyperdbValueError, message:
+                    raise UsageError, message
 
             # try the set
             try:
@@ -623,32 +612,15 @@ Command help:
         # handle the propname=value argument
         props = self.props_from_args(args[1:])
 
-        # if the value isn't a number, look up the linked class to get the
-        # number
+        # convert the user-input value to a value used for find()
         for propname, value in props.items():
-            num_re = re.compile('^\d+$')
-            if value == '-1':
-                props[propname] = None
-            elif not num_re.match(value):
-                # get the property
-                try:
-                    property = cl.properties[propname]
-                except KeyError:
-                    raise UsageError, _('%(classname)s has no property '
-                        '"%(propname)s"')%locals()
-
-                # make sure it's a link
-                if (not isinstance(property, hyperdb.Link) and not
-                        isinstance(property, hyperdb.Multilink)):
-                    raise UsageError, _('You may only "find" link properties')
-
-                # get the linked-to class and look up the key property
-                link_class = self.db.getclass(property.classname)
-                try:
-                    props[propname] = link_class.lookup(value)
-                except TypeError:
-                    raise UsageError, _('%(classname)s has no key property"')%{
-                        'classname': link_class.classname}
+            if ',' in value:
+                values = value.split(',')
+            else:
+                values = []
+            d = props[propname] = {}
+            for value in values:
+                d[hyperdb.rawToHyperdb(self.db, cl, None, propname, value)] = 1
 
         # now do the find 
         try:
@@ -770,39 +742,11 @@ Command help:
 
         # convert types
         for propname, value in props.items():
-            # get the property
             try:
-                proptype = properties[propname]
-            except KeyError:
-                raise UsageError, _('%(classname)s has no property '
-                    '"%(propname)s"')%locals()
-
-            if isinstance(proptype, hyperdb.Date):
-                try:
-                    props[propname] = date.Date(value)
-                except ValueError, message:
-                    raise UsageError, _('"%(value)s": %(message)s')%locals()
-            elif isinstance(proptype, hyperdb.Interval):
-                try:
-                    props[propname] = date.Interval(value)
-                except ValueError, message:
-                    raise UsageError, _('"%(value)s": %(message)s')%locals()
-            elif isinstance(proptype, hyperdb.Password):
-                m = pwre.match(value)
-                if m:
-                    # password is being given to us encrypted
-                    p = password.Password()
-                    p.scheme = m.group(1)
-                    p.password = m.group(2)
-                    props[propname] = p
-                else:
-                    props[propname] = password.Password(value)
-            elif isinstance(proptype, hyperdb.Multilink):
-                props[propname] = value.split(',')
-            elif isinstance(proptype, hyperdb.Boolean):
-                props[propname] = value.lower() in ('yes', 'true', 'on', '1')
-            elif isinstance(proptype, hyperdb.Number):
-                props[propname] = float(value)
+                props[propname] = hyperdb.rawToHyperdb(self.db, cl, None,
+                    propname, value)
+            except hyperdb.HyperdbValueError, message:
+                raise UsageError, message
 
         # check for the key property
         propname = cl.getkey()
@@ -878,6 +822,7 @@ Command help:
         specified, all properties are displayed. By default, the column widths
         are the width of the largest value. The width may be explicitly defined
         by defining the property as "name:width". For example::
+
           roundup> table priority id,name:10
           Id Name
           1  fatal-bug 
@@ -886,7 +831,8 @@ Command help:
           4  feature   
 
         Also to make the width of the column the width of the label,
-        leave a trailing : without a width on the property. E.G.
+        leave a trailing : without a width on the property. For example::
+
           roundup> table priority id,name:
           Id Name
           1  fata
@@ -984,7 +930,7 @@ Command help:
 
     def do_commit(self, args):
         '''Usage: commit
-        Commit all changes made to the database.
+        Commit changes made to the database during an interactive session.
 
         The changes made during an interactive session are not
         automatically written to the database - they must be committed
@@ -1057,9 +1003,11 @@ Command help:
         '''Usage: export [class[,class]] export_dir
         Export the database to colon-separated-value files.
 
+        Optionally limit the export to just the names classes.
+
         This action exports the current data from the database into
         colon-separated-value files that are placed in the nominated
-        destination directory. The journals are not exported.
+        destination directory.
         '''
         # grab the directory to export to
         if len(args) < 1:
@@ -1078,33 +1026,44 @@ Command help:
         # do all the classes specified
         for classname in classes:
             cl = self.get_class(classname)
+
             f = open(os.path.join(dir, classname+'.csv'), 'w')
             writer = rcsv.writer(f, rcsv.colon_separated)
+
             properties = cl.getprops()
-            propnames = properties.keys()
-            propnames.sort()
+            propnames = cl.export_propnames()
             fields = propnames[:]
             fields.append('is retired')
             writer.writerow(fields)
 
-            # all nodes for this class (not using list() 'cos it doesn't
-            # include retired nodes)
-
-            for nodeid in self.db.getclass(classname).getnodeids():
-                # get the regular props
+            # all nodes for this class
+            for nodeid in cl.getnodeids():
                 writer.writerow(cl.export_list(propnames, nodeid))
+                if hasattr(cl, 'export_files'):
+                    cl.export_files(dir, nodeid)
 
             # close this file
             f.close()
+
+            # export the journals
+            jf = open(os.path.join(dir, classname+'-journals.csv'), 'w')
+            journals = rcsv.writer(jf, rcsv.colon_separated)
+            map(journals.writerow, cl.export_journals())
+            jf.close()
         return 0
 
     def do_import(self, args):
         '''Usage: import import_dir
-        Import a database from the directory containing CSV files, one per
+        Import a database from the directory containing CSV files, two per
         class to import.
 
-        The files must define the same properties as the class (including having
-        a "header" line with those property names.)
+        The files used in the import are:
+
+        <class>.csv
+          This must define the same properties as the class (including
+          having a "header" line with those property names.)
+        <class>-journals.csv
+          This defines the journals for the items being imported.
 
         The imported nodes will have the same nodeid as defined in the
         import file, thus replacing any existing content.
@@ -1119,44 +1078,46 @@ Command help:
             raise UsageError, _(rcsv.error)
         from roundup import hyperdb
 
-        for file in os.listdir(args[0]):
+        # directory to import from
+        dir = args[0]
+
+        # import all the files
+        for file in os.listdir(dir):
+            classname, ext = os.path.splitext(file)
             # we only care about CSV files
-            if not file.endswith('.csv'):
+            if ext != '.csv' or classname.endswith('-journals'):
                 continue
 
-            f = open(os.path.join(args[0], file))
-
-            # get the classname
-            classname = os.path.splitext(file)[0]
+            cl = self.get_class(classname)
 
             # ensure that the properties and the CSV file headings match
-            cl = self.get_class(classname)
+            f = open(os.path.join(dir, file))
             reader = rcsv.reader(f, rcsv.colon_separated)
             file_props = None
             maxid = 1
-
-# XXX we don't _really_ need to do this...
-#            properties = cl.getprops()
-#            propnames = properties.keys()
-#            propnames.sort()
-#            m = file_props[:]
-#            m.sort()
-#            if m != propnames:
-#                raise UsageError, _('Import file doesn\'t define the same '
-#                    'properties as "%(arg0)s".')%{'arg0': args[0]}
-
             # loop through the file and create a node for each entry
             for r in reader:
                 if file_props is None:
                     file_props = r
                     continue
-
                 # do the import and figure the current highest nodeid
-                maxid = max(maxid, int(cl.import_list(file_props, r)))
+                nodeid = int(cl.import_list(file_props, r))
+                if hasattr(cl, 'import_files'):
+                    cl.import_files(dir, nodeid)
+                maxid = max(maxid, nodeid)
+
+            f.close()
+
+            # import the journals
+            f = open(os.path.join(args[0], classname + '-journals.csv'))
+            reader = rcsv.reader(f, rcsv.colon_separated)
+            cl.import_journals(reader)
+            f.close()
 
             # set the id counter
             print 'setting', classname, maxid+1
             self.db.setid(classname, str(maxid+1))
+
         return 0
 
     def do_pack(self, args):
@@ -1352,7 +1313,7 @@ Date format is "YYYY-MM-DD" eg:
 
     def main(self):
         try:
-            opts, args = getopt.getopt(sys.argv[1:], 'i:u:hcdsS:')
+            opts, args = getopt.getopt(sys.argv[1:], 'i:u:hcdsS:v')
         except getopt.GetoptError, e:
             self.usage(str(e))
             return 1
@@ -1372,6 +1333,9 @@ Date format is "YYYY-MM-DD" eg:
             if opt == '-h':
                 self.usage()
                 return 0
+            if opt == '-v':
+                print '%s (python %s)'%(roundup_version, sys.version.split()[0])
+                return 0
             if opt == '-i':
                 self.tracker_home = arg
             if opt == '-c':
@@ -1406,7 +1370,6 @@ Date format is "YYYY-MM-DD" eg:
             if self.db:
                 self.db.close()
 
-
 if __name__ == '__main__':
     tool = AdminTool()
     sys.exit(tool.main())
