@@ -3,9 +3,9 @@
 # 
 # $Id$
 
-__doc__ = """
-Extended CGI traceback handler by Ka-Ping Yee, <ping@lfw.org>.
+"""Extended CGI traceback handler by Ka-Ping Yee, <ping@lfw.org>.
 """
+__docformat__ = 'restructuredtext'
 
 import sys, os, types, string, keyword, linecache, tokenize, inspect, cgi
 import pydoc, traceback
