@@ -17,11 +17,12 @@
 # 
 # $Id$
 
-__doc__ = """
-Date, time and time interval handling.
+"""Date, time and time interval handling.
 """
+__docformat__ = 'restructuredtext'
 
 import time, re, calendar, types
+from types import *
 from i18n import _
 
 def _add_granularity(src, order, value = 1):
@@ -51,6 +52,7 @@ class Date:
     care of these conversions. In the following examples, suppose that yyyy
     is the current year, mm is the current month, and dd is the current day
     of the month; and suppose that the user is on Eastern Standard Time.
+    Examples::
 
       "2000-04-17" means <Date 2000-04-17.00:00:00>
       "01-25" means <Date yyyy-01-25.00:00:00>
@@ -69,9 +71,8 @@ class Date:
     separately. For example, when evaluating "2000-06-25 + 1m 10d", we
     first add one month to get 2000-07-25, then add 10 days to get
     2000-08-04 (rather than trying to decide whether 1m 10d means 38 or 40
-    or 41 days).
+    or 41 days).  Example usage::
 
-    Example usage:
         >>> Date(".")
         <Date 2000-06-26.00:34:02>
         >>> _.local(-5)
@@ -90,32 +91,58 @@ class Date:
     The date format 'yyyymmddHHMMSS' (year, month, day, hour,
     minute, second) is the serialisation format returned by the serialise()
     method, and is accepted as an argument on instatiation.
+
+    The date class handles basic arithmetic::
+
+        >>> d1=Date('.')
+        >>> d1
+        <Date 2004-04-06.22:04:20.766830>
+        >>> d2=Date('2003-07-01')
+        >>> d2
+        <Date 2003-07-01.00:00:0.000000>
+        >>> d1-d2
+        <Interval + 280d 22:04:20>
+        >>> i1=_
+        >>> d2+i1
+        <Date 2004-04-06.22:04:20.000000>
+        >>> d1-i1
+        <Date 2003-07-01.00:00:0.000000>
     '''
-    
+
     def __init__(self, spec='.', offset=0, add_granularity=0):
         """Construct a date given a specification and a time zone offset.
 
-          'spec' is a full date or a partial form, with an optional
-                 added or subtracted interval. Or a date 9-tuple.
-        'offset' is the local time zone offset from GMT in hours.
+        'spec'
+           is a full date or a partial form, with an optional added or
+           subtracted interval. Or a date 9-tuple.
+        'offset'
+           is the local time zone offset from GMT in hours.
         """
         if type(spec) == type(''):
             self.set(spec, offset=offset, add_granularity=add_granularity)
-        else:
+            return
+        elif hasattr(spec, 'tuple'):
+            spec = spec.tuple()
+        try:
             y,m,d,H,M,S,x,x,x = spec
+            frac = S - int(S)
             ts = calendar.timegm((y,m,d,H+offset,M,S,0,0,0))
             self.year, self.month, self.day, self.hour, self.minute, \
                 self.second, x, x, x = time.gmtime(ts)
+            # we lost the fractional part
+            self.second = self.second + frac
+        except:
+            raise ValueError, 'Unknown spec %r'%spec
 
-    usagespec='[yyyy]-[mm]-[dd].[H]H:MM[:SS][offset]'
+    usagespec='[yyyy]-[mm]-[dd].[H]H:MM[:SS.SSS][offset]'
     def set(self, spec, offset=0, date_re=re.compile(r'''
             ((?P<y>\d\d\d\d)([/-](?P<m>\d\d?)([/-](?P<d>\d\d?))?)? # yyyy[-mm[-dd]]
             |(?P<a>\d\d?)[/-](?P<b>\d\d?))?              # or mm-dd
             (?P<n>\.)?                                     # .
-            (((?P<H>\d?\d):(?P<M>\d\d))?(:(?P<S>\d\d))?)?  # hh:mm:ss
+            (((?P<H>\d?\d):(?P<M>\d\d))?(:(?P<S>\d\d(\.\d+)?))?)?  # hh:mm:ss
             (?P<o>.+)?                                     # offset
             ''', re.VERBOSE), serialised_re=re.compile(r'''
-            (\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)
+            (\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d?(\.\d+)?)
             ''', re.VERBOSE), add_granularity=0):
         ''' set the date to the value in spec
         '''
@@ -123,8 +150,10 @@ class Date:
         m = serialised_re.match(spec)
         if m is not None:
             # we're serialised - easy!
-            self.year, self.month, self.day, self.hour, self.minute, \
-                self.second = map(int, m.groups()[:6])
+            g = m.groups()
+            (self.year, self.month, self.day, self.hour, self.minute) = \
+                map(int, g[:5])
+            self.second = float(g[5])
             return
 
         # not serialised data, try usual format
@@ -138,7 +167,11 @@ class Date:
             _add_granularity(info, 'SMHdmyab')
 
         # get the current date as our default
-        y,m,d,H,M,S,x,x,x = time.gmtime(time.time())
+        ts = time.time()
+        frac = ts - int(ts)
+        y,m,d,H,M,S,x,x,x = time.gmtime(ts)
+        # gmtime loses the fractional seconds 
+        S = S + frac
 
         if info['y'] is not None or info['a'] is not None:
             if info['y'] is not None:
@@ -159,21 +192,26 @@ class Date:
             H = int(info['H']) - offset
             M = int(info['M'])
             S = 0
-            if info['S'] is not None: S = int(info['S'])
+            if info['S'] is not None:
+                S = float(info['S'])
 
         if add_granularity:
             S = S - 1
         
         # now handle the adjustment of hour
+        frac = S - int(S)
         ts = calendar.timegm((y,m,d,H,M,S,0,0,0))
         self.year, self.month, self.day, self.hour, self.minute, \
             self.second, x, x, x = time.gmtime(ts)
+        # we lost the fractional part along the way
+        self.second = self.second + frac
 
         if info.get('o', None):
             try:
                 self.applyInterval(Interval(info['o'], allowdate=0))
             except ValueError:
-                raise ValueError, _('Not a date spec: %s' % self.usagespec)
+                raise ValueError, _('%r not a date spec (%s)')%(spec,
+                    self.usagespec)
 
     def addInterval(self, interval):
         ''' Add the interval to this date, returning the date tuple
@@ -185,7 +223,8 @@ class Date:
         day = self.day + sign * interval.day
         hour = self.hour + sign * interval.hour
         minute = self.minute + sign * interval.minute
-        second = self.second + sign * interval.second
+        # Intervals work on whole seconds
+        second = int(self.second) + sign * interval.second
 
         # now cope with under- and over-flow
         # first do the time
@@ -228,6 +267,7 @@ class Date:
 
     def differenceDate(self, other):
         "Return the difference between this date and another date"
+        return self - other
 
     def applyInterval(self, interval):
         ''' Apply the interval to this date
@@ -265,7 +305,8 @@ class Date:
             self.minute, self.second, 0, 0, 0))
         b = calendar.timegm((other.year, other.month, other.day,
             other.hour, other.minute, other.second, 0, 0, 0))
-        diff = a - b
+        # intervals work in whole seconds
+        diff = int(a - b)
         if diff > 0:
             sign = 1
         else:
@@ -277,21 +318,29 @@ class Date:
         d = diff/(24*60*60)
         return Interval((0, 0, d, H, M, S), sign=sign)
 
-    def __cmp__(self, other):
+    def __cmp__(self, other, int_seconds=0):
         """Compare this date to another date."""
         if other is None:
             return 1
-        for attr in ('year', 'month', 'day', 'hour', 'minute', 'second'):
+        for attr in ('year', 'month', 'day', 'hour', 'minute'):
             if not hasattr(other, attr):
                 return 1
             r = cmp(getattr(self, attr), getattr(other, attr))
             if r: return r
-        return 0
+        if not hasattr(other, 'second'):
+            return 1
+        if int_seconds:
+            return cmp(int(self.second), int(other.second))
+        return cmp(self.second, other.second)
 
     def __str__(self):
         """Return this date as a string in the yyyy-mm-dd.hh:mm:ss format."""
-        return '%4d-%02d-%02d.%02d:%02d:%02d'%(self.year, self.month, self.day,
-            self.hour, self.minute, self.second)
+        return self.formal()
+
+    def formal(self, sep='.', sec='%02d'):
+        f = '%%4d-%%02d-%%02d%s%%02d:%%02d:%s'%(sep, sec)
+        return f%(self.year, self.month, self.day, self.hour, self.minute,
+            self.second)
 
     def pretty(self, format='%d %B %Y'):
         ''' print up the date date using a pretty format...
@@ -300,14 +349,14 @@ class Date:
             format, then the day number will be removed from output.
         '''
         str = time.strftime(format, (self.year, self.month, self.day,
-            self.hour, self.minute, self.second, 0, 0, 0))
+            self.hour, self.minute, int(self.second), 0, 0, 0))
         # handle zero day by removing it
         if format.startswith('%d') and str[0] == '0':
             return ' ' + str[1:]
         return str
 
     def __repr__(self):
-        return '<Date %s>'%self.__str__()
+        return '<Date %s>'%self.formal(sec='%f')
 
     def local(self, offset):
         """ Return this date as yyyy-mm-dd.hh:mm:ss in a local time zone.
@@ -320,9 +369,17 @@ class Date:
             self.second, 0, 0, 0)
 
     def serialise(self):
-        return '%4d%02d%02d%02d%02d%02d'%(self.year, self.month,
+        return '%4d%02d%02d%02d%02d%f'%(self.year, self.month,
             self.day, self.hour, self.minute, self.second)
 
+    def timestamp(self):
+        ''' return a UNIX timestamp for this date '''
+        frac = self.second - int(self.second)
+        ts = calendar.timegm((self.year, self.month, self.day, self.hour,
+            self.minute, self.second, 0, 0, 0))
+        # we lose the fractional part
+        return ts + frac
+
 class Interval:
     '''
     Date intervals are specified using the suffixes "y", "m", and "d". The
@@ -377,17 +434,21 @@ class Interval:
     '''
     def __init__(self, spec, sign=1, allowdate=1, add_granularity=0):
         """Construct an interval given a specification."""
-        if type(spec) == type(''):
+        if type(spec) in (IntType, FloatType, LongType):
+            self.from_seconds(spec)
+        elif type(spec) in (StringType, UnicodeType):
             self.set(spec, allowdate=allowdate, add_granularity=add_granularity)
         else:
             if len(spec) == 7:
                 self.sign, self.year, self.month, self.day, self.hour, \
                     self.minute, self.second = spec
+                self.second = int(self.second)
             else:
                 # old, buggy spec form
                 self.sign = sign
                 self.year, self.month, self.day, self.hour, self.minute, \
                     self.second = spec
+                self.second = int(self.second)
 
     def set(self, spec, allowdate=1, interval_re=re.compile('''
             \s*(?P<s>[-+])?         # + or -
@@ -475,6 +536,8 @@ class Interval:
             l.append('%d:%02d'%(self.hour, self.minute))
         if l:
             l.insert(0, {1:'+', -1:'-'}[self.sign])
+        else:
+            l.append('00:00')
         return ' '.join(l)
 
     def __add__(self, other):
@@ -514,11 +577,11 @@ class Interval:
         raise TypeError, "Can't add %r"%other
 
     def __div__(self, other):
-        ''' Divide this interval by an int value.
+        """ Divide this interval by an int value.
 
             Can't divide years and months sensibly in the _same_
             calculation as days/time, so raise an error in that situation.
-        '''
+        """
         try:
             other = float(other)
         except TypeError:
@@ -564,9 +627,9 @@ class Interval:
         '''
         if self.year:
             if self.year == 1:
-                return _('1 year')
+                s = _('1 year')
             else:
-                return _('%(number)s years')%{'number': self.year}
+                s = _('%(number)s years')%{'number': self.year}
         elif self.month or self.day > 13:
             days = (self.month * 30) + self.day
             if days > 28:
@@ -622,14 +685,50 @@ class Interval:
         return '%s%04d%02d%02d%02d%02d%02d'%(sign, self.year, self.month,
             self.day, self.hour, self.minute, self.second)
 
+    def as_seconds(self):
+        '''Calculate the Interval as a number of seconds.
+        
+        Months are counted as 30 days, years as 365 days. Returns a Long
+        int.
+        '''
+        n = self.year * 365L
+        n = n + self.month * 30
+        n = n + self.day
+        n = n * 24
+        n = n + self.hour
+        n = n * 60
+        n = n + self.minute
+        n = n * 60
+        n = n + self.second
+        return n * self.sign
+
+    def from_seconds(self, val):
+        '''Figure my second, minute, hour and day values using a seconds
+        value.
+        '''
+        if val < 0:
+            self.sign = -1
+            val = -val
+        else:
+            self.sign = 1
+        self.second = val % 60
+        val = val / 60
+        self.minute = val % 60
+        val = val / 60
+        self.hour = val % 24
+        val = val / 24
+        self.day = val
+        self.month = self.year = 0
+
+
 def fixTimeOverflow(time):
-    ''' Handle the overflow in the time portion (H, M, S) of "time":
+    """ Handle the overflow in the time portion (H, M, S) of "time":
             (sign, y,m,d,H,M,S)
 
         Overflow and underflow will at most affect the _days_ portion of
         the date. We do not overflow days to months as we don't know _how_
         to, generally.
-    '''
+    """
     # XXX we could conceivably use this function for handling regular dates
     # XXX too - we just need to interrogate the month/year for the day
     # XXX overflow...
@@ -655,20 +754,24 @@ def fixTimeOverflow(time):
     return (sign, y, m, d, H, M, S)
 
 class Range:
-    """
-    Represents range between two values
+    """Represents range between two values
     Ranges can be created using one of theese two alternative syntaxes:
         
-        1. Native english syntax: 
-            [[From] <value>][ To <value>]
-           Keywords "From" and "To" are case insensitive. Keyword "From" is optional.
+    1. Native english syntax::
 
-        2. "Geek" syntax:
-            [<value>][; <value>]
+            [[From] <value>][ To <value>]
+
+       Keywords "From" and "To" are case insensitive. Keyword "From" is
+       optional.
+
+    2. "Geek" syntax::
+
+          [<value>][; <value>]
 
     Either first or second <value> can be omitted in both syntaxes.
 
-    Examples (consider local time is Sat Mar  8 22:07:48 EET 2003):
+    Examples (consider local time is Sat Mar  8 22:07:48 EET 2003)::
+
         >>> Range("from 2-12 to 4-2")
         <Range from 2003-02-12.00:00:00 to 2003-04-02.00:00:00>
         
