@@ -17,11 +17,11 @@
 # 
 # $Id$
 
-__doc__ = '''
-Tracker handling (open tracker).
+'''Tracker handling (open tracker).
 
 Backwards compatibility for the old-style "imported" trackers.
 '''
+__docformat__ = 'restructuredtext'
 
 import os
 
@@ -43,8 +43,8 @@ class Tracker:
         self._load_config('security.py', db=db)
 
 
-    def __load_python(self, file):
-        file = os.path.join(tracker_home, file)
+    def _load_python(self, file):
+        file = os.path.join(self.tracker_home, file)
         vars = Vars()
         execfile(file, vars.__dict__)
         return vars
