@@ -17,9 +17,9 @@
 # 
 # $Id$
 
-__doc__ = """
-Password handling (encoding, decoding).
+"""Password handling (encoding, decoding).
 """
+__docformat__ = 'restructuredtext'
 
 import sha, re, string, random
 try:
@@ -28,6 +28,10 @@ except:
     crypt = None
     pass
 
+class PasswordValueError(ValueError):
+    ''' The password value is not valid '''
+    pass
+
 def encodePassword(plaintext, scheme, other=None):
     '''Encrypt the plaintext password.
     '''
@@ -45,7 +49,7 @@ def encodePassword(plaintext, scheme, other=None):
     elif scheme == 'plaintext':
         s = plaintext
     else:
-        raise ValueError, 'Unknown encryption scheme "%s"'%scheme
+        raise PasswordValueError, 'unknown encryption scheme %r'%scheme
     return s
 
 def generatePassword(length=8):
@@ -78,13 +82,15 @@ class Password:
     default_scheme = 'SHA'        # new encryptions use this scheme
     pwre = re.compile(r'{(\w+)}(.+)')
 
-    def __init__(self, plaintext=None, scheme=None):
+    def __init__(self, plaintext=None, scheme=None, encrypted=None):
         '''Call setPassword if plaintext is not None.'''
         if scheme is None:
             scheme = self.default_scheme
         if plaintext is not None:
             self.password = encodePassword(plaintext, self.default_scheme)
             self.scheme = self.default_scheme
+        elif encrypted is not None:
+            self.unpack(encrypted)
         else:
             self.password = None
             self.scheme = self.default_scheme
