@@ -16,8 +16,13 @@
 # 
 # $Id$
 
+"""Command-line script stub that calls the roundup.mailgw.
+"""
+__docformat__ = 'restructuredtext'
+
 # python version check
 from roundup import version_check
+from roundup import __version__ as roundup_version
 
 import sys, os, re, cStringIO, getopt
 
@@ -27,14 +32,18 @@ from roundup.i18n import _
 def usage(args, message=None):
     if message is not None:
         print message
-    print _('Usage: %(program)s [[-C class] -S field=value]* <instance '
+    print _('Usage: %(program)s [-v] [[-C class] -S field=value]* <instance '
         'home> [method]')%{'program': args[0]}
     print _('''
+Options:
+ -v: print version and exit
+ -C / -S: see below
 
-The roundup mail gateway may be called in one of three ways:
+The roundup mail gateway may be called in one of four ways:
  . with an instance home as the only argument,
  . with both an instance home and a mail spool file, or
- . with both an instance home and a pop server account.
+ . with both an instance home and a POP/APOP server account.
+ . with both an instance home and a IMAP/IMAPS server account.
  
 It also supports optional -C and -S arguments that allows you to set a
 fields for a class created by the roundup-mailgw. The default class if
@@ -72,6 +81,19 @@ APOP:
  Same as POP, but using Authenticated POP:
     apop username:password@server
 
+IMAP:
+ Connect to an IMAP server. This supports the same notation as that of
+ POP mail.
+    imap username:password@server
+ It also allows you to specify a specific mailbox other than INBOX using
+ this format:
+    imap username:password@server mailbox
+ 
+IMAPS:
+ Connect to an IMAP server over ssl. 
+ This supports the same notation as IMAP.
+    imaps username:password@server [mailbox]
+ 
 ''')
     return 1
 
@@ -81,12 +103,18 @@ def main(argv):
     # take the argv array and parse it leaving the non-option
     # arguments in the args array.
     try:
-        optionsList, args = getopt.getopt(argv[1:], 'C:S:', ['set=', 'class='])
+        optionsList, args = getopt.getopt(argv[1:], 'vC:S:', ['set=',
+            'class='])
     except getopt.GetoptError:
         # print help information and exit:
         usage(argv)
         sys.exit(2)
 
+    for (opt, arg) in optionsList:
+        if opt == '-v':
+            print '%s (python %s)'%(roundup_version, sys.version.split()[0])
+            return
+
     # figure the instance home
     if len(args) > 0:
         instance_home = args[0]
@@ -113,7 +141,7 @@ def main(argv):
         # otherwise, figure what sort of mail source to handle
         if len(args) < 3:
             return usage(argv, _('Error: not enough source specification information'))
-        source, specification = args[1:]
+        source, specification = args[1:3]
         if source == 'mailbox':
             return handler.do_mailbox(specification)
         elif source == 'pop':
@@ -130,10 +158,23 @@ def main(argv):
                 return handler.do_apop(m.group('server'), m.group('user'),
                     m.group('pass'))
             return usage(argv, _('Error: apop specification not valid'))
+        elif source == 'imap' or source == 'imaps':
+            m = re.match(r'((?P<user>[^:]+)(:(?P<pass>.+))?@)?(?P<server>.+)',
+                specification)
+            if m:
+                ssl = 0
+                if source == 'imaps':
+                    ssl = 1
+                mailbox = ''
+                if len(args) > 3:
+                    mailbox = args[3]
+                return handler.do_imap(m.group('server'), m.group('user'),
+                    m.group('pass'), mailbox, ssl)
 
-        return usage(argv, _('Error: The source must be either "mailbox", "pop" or "apop"'))
+        return usage(argv, _('Error: The source must be either "mailbox",'
+            ' "pop", "apop", "imap" or "imaps"'))
     finally:
-        db.close()
+        handler.db.close()
 
 def run():
     # time out after a minute if we can
