@@ -14,22 +14,30 @@
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 # 
-""" HTTP Server that serves roundup.
+
+"""Command-line script that runs a server over roundup.cgi.client.
 
 $Id$
 """
+__docformat__ = 'restructuredtext'
 
 # python version check
 from roundup import version_check
+from roundup import __version__ as roundup_version
 
 import sys, os, urllib, StringIO, traceback, cgi, binascii, getopt, imp
-import BaseHTTPServer, socket
+import SocketServer, BaseHTTPServer, socket, errno
 
 # Roundup modules of use here
 from roundup.cgi import cgitb, client
 import roundup.instance
 from roundup.i18n import _
 
+try:
+    import signal
+except:
+    signal = None
+
 #
 ##  Configuration
 #
@@ -56,16 +64,20 @@ LOGFILE = None
 ##  end configuration
 #
 
+# "default" favicon.ico
+# generate by using "icotool" and tools/base64
 import zlib, base64
 favico = zlib.decompress(base64.decodestring('''
-eJyVUk2IQVEUfn4yaRYjibdQZiVba/ZE2djIUmHWFjaKGVmIlY2iFMVG2ViQhXqFSP6iFFJvw4uF
-LGdWd743mpeMn+a88917Oue7955z3qEoET6FQkHx8iahKDV2A8B7XgERRf/EKMSUzyf8ypbbnnQy
-mWBdr9eVSkVw3tJGoxGNRpvNZigUyufzWPv9Pvwcx0UiERj7/V4g73Y7j8fTarWMRmO73U4kEkKI
-YZhardbr9eLxuOD0+/2ZTMZisYjFYpqmU6kU799uN5tNMBg8HA7ZbPY8GaTh8/mEipRKpclk0ul0
-NpvNarUmk0mWZS/yr9frcrmc+iMOh+NWydPp1Ov1SiSSc344HL7fKKfTiSN2u12tVqOcxWJxn6/V
-ag0GAwxkrlKp5vP5fT7ulMlk6XRar9dLpVIUXi6Xb5Hxa1wul0ajKZVKsVjM7XYXCoVOp3OVPJvN
-AoFAtVo1m825XO7hSODOYrH4kHbxxGAwwODBGI/H6DBs5LNara7yl8slGjIcDsHpdrunU6PRCAP2
-r3fPdUcIYeyEfLSAJ0LeAUZHCAt8Al/8/kLIEWDB5YDj0wm8fAP6fVfo
+eJztjr1PmlEUh59XgVoshdYPWorFIhaRFq0t9pNq37b60lYSTRzcTFw6GAfj5gDYaF0dTB0MxMSE
+gQQd3FzKJiEC0UCIUUN1M41pV2JCXySg/0ITn5tfzvmdc+85FwT56HSc81UJjXJsk1UsNcsSqCk1
+BS64lK+vr7OyssLJyQl2ux2j0cjU1BQajYZIJEIwGMRms+H3+zEYDExOTjI2Nsbm5iZWqxWv18vW
+1hZDQ0Ok02kmJiY4Ojpienqa3d1dxsfHUSqVeDwe5ufnyeVyrK6u4nK5ODs7Y3FxEYfDwdzcHCaT
+icPDQ5LJJIIgMDIyQj6fZ39/n+3tbdbW1pAkiYWFBWZmZtjb2yMejzM8PEwgEMDn85HNZonFYqjV
+asLhMMvLy2QyGfR6PaOjowwODmKxWDg+PkalUhEKhSgUCiwtLWE2m9nZ2UGhULCxscHp6SmpVIpo
+NMrs7CwHBwdotVoSiQRXXPG/IzY7RHtt922xjFRb01H1XhKfPBNbi/7my7rrLXJ88eppvxwEfV3f
+NY3Y6exofVdsV3+2wnPFDdPjB83n7xuVpcFvygPbGwxF31LZIKrQDfR2Xvh7lmrX654L/7bvlnng
+bn3Zuj8M9Hepux6VfZtW1yA6K7cfGqVu8TL325u+fHTb71QKbk+7TZQ+lTc6RcnpqW8qmVQBoj/g
+23eo0sr/NIGvB37K+lOWXMvJ+uWFeKGU/03Cb7n3D4M3wxI=
 '''.strip()))
 
 class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
@@ -82,11 +94,11 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             self.inner_run_cgi()
         except client.NotFound:
             self.send_error(404, self.path)
-        except client.Unauthorised:
-            self.send_error(403, self.path)
+        except client.Unauthorised, message:
+            self.send_error(403, '%s (%s)'%(self.path, message))
         except:
             exc, val, tb = sys.exc_info()
-            if hasattr(socket, 'timeout') and exc == socket.timeout:
+            if hasattr(socket, 'timeout') and isinstance(val, socket.timeout):
                 self.log_error('timeout')
             else:
                 # it'd be nice to be able to detect if these are going to have
@@ -131,12 +143,11 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
         rest = self.path
 
         if rest == '/favicon.ico':
-            raise client.NotFound
-#            self.send_response(200)
-#            self.send_header('Content-Type', 'image/x-ico')
-#            self.end_headers()
-#            self.wfile.write(favicon)
-#            return
+            self.send_response(200)
+            self.send_header('Content-Type', 'image/x-icon')
+            self.end_headers()
+            self.wfile.write(favico)
+            return
 
         i = rest.rfind('?')
         if i >= 0:
@@ -223,11 +234,23 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             # stderr is no longer viable
             pass
 
+def error():
+    exc_type, exc_value = sys.exc_info()[:2]
+    return _('Error: %s: %s' % (exc_type, exc_value))
+
+# See whether we can use the forking server
+if hasattr(os, 'fork'):
+    class server_class(SocketServer.ForkingMixIn, BaseHTTPServer.HTTPServer):
+        pass
+else:
+    server_class = BaseHTTPServer.HTTPServer
+
 try:
     import win32serviceutil
 except:
     RoundupService = None
 else:
+
     # allow the win32
     import win32service
     import win32event
@@ -318,37 +341,45 @@ else:
                     break
             return rv
 
-
 def usage(message=''):
-    if message:
-        message = _('Error: %(error)s\n\n')%{'error': message}
     if RoundupService:
-        win = ''' -c: Windows Service options.  If you want to run the server as a Windows
-     Service, you must configure the rest of the options by changing the
-     constants of this program.  You will at least configure one tracker
-     in the TRACKER_HOMES variable.  This option is mutually exclusive
-     from the rest.  Typing "roundup-server -c help" shows Windows
-     Services specifics.'''
+        os_part = \
+""''' -c <Command>  Windows Service options.
+               If you want to run the server as a Windows Service, you must
+               configure the rest of the options by changing the constants
+               of this program.  You will at least configure one tracker in
+               the TRACKER_HOMES variable.  This option is mutually exclusive
+               from the rest.  Typing "roundup-server -c help" shows Windows
+               Services specifics.'''
     else:
-        win = ''
-    print _('''%(message)sUsage:
-roundup-server [options] [name=tracker home]*
+        os_part = ''' -u <UID>      runs the Roundup web server as this UID
+ -g <GID>      runs the Roundup web server as this GID
+ -d <PIDfile>  run the server in the background and write the server's PID
+               to the file indicated by PIDfile. The -l option *must* be
+               specified if -d is used.'''
+    port=PORT
+    if message:
+        message += '\n'
+    print _('''%(message)sUsage: roundup-server [options] [name=tracker home]*
 
-options:
- -n: sets the host name
- -p: sets the port to listen on
- -l: sets a filename to log to (instead of stdout)
- -d: run the server in the background and on UN*X write the server's PID
-     to the nominated file. The -l option *must* be specified if this
-     option is.
- -N: log client machine names in access log instead of IP addresses (much
-     slower)
-%(win)s
+Options:
+ -v            prints the Roundup version number and exits
+ -n <name>     sets the host name of the Roundup web server instance
+ -p <port>     sets the port to listen on (default: %(port)s)
+ -l <fname>    log to the file indicated by fname instead of stderr/stdout
+ -N            log client machine names instead of IP addresses (much slower)
+%(os_part)s
 
-name=tracker home:
-   Sets the tracker home(s) to use. The name is how the tracker is
-   identified in the URL (it's the first part of the URL path). The
-   tracker home is the directory that was identified when you did
+Examples:
+ roundup-server support=/var/spool/roundup-trackers/support
+
+ roundup-server -d /var/run/roundup.pid -l /var/log/roundup.log \\
+     support=/var/spool/roundup-trackers/support
+
+How to use "name=tracker home":
+   These arguments set the tracker home(s) to use. The name is how the
+   tracker is identified in the URL (it's the first part of the URL path).
+   The tracker home is the directory that was identified when you did
    "roundup-admin init". You may specify any number of these name=home
    pairs on the command-line. For convenience, you may edit the
    TRACKER_HOMES variable in the roundup-server file instead.
@@ -380,9 +411,9 @@ def daemonize(pidfile):
         pidfile = open(pidfile, 'w')
         pidfile.write(str(pid))
         pidfile.close()
-        os._exit(0)         
+        os._exit(0)
 
-    os.chdir("/")         
+    os.chdir("/")
     os.umask(0)
 
     # close off sys.std(in|out|err), redirect to devnull so the file
@@ -392,16 +423,7 @@ def daemonize(pidfile):
     os.dup2(devnull, 1)
     os.dup2(devnull, 2)
 
-def abspath(path):
-    ''' Make the given path an absolute path.
-
-        Code from Zope-Coders posting of 2002-10-06 by GvR.
-    '''
-    if not os.path.isabs(path):
-        path = os.path.join(os.getcwd(), path)
-    return os.path.normpath(path)
-
-def run():
+def run(port=PORT, success_message=None):
     ''' Script entry point - handle args and figure out what to to.
     '''
     # time out after a minute if we can
@@ -410,7 +432,6 @@ def run():
         socket.setdefaulttimeout(60)
 
     hostname = HOSTNAME
-    port = PORT
     pidfile = PIDFILE
     logfile = LOGFILE
     user = ROUNDUP_USER
@@ -419,7 +440,7 @@ def run():
 
     try:
         # handle the command-line args
-        options = 'n:p:u:d:l:hN'
+        options = 'n:p:g:u:d:l:hNv'
         if RoundupService:
             options += 'c'
 
@@ -432,10 +453,14 @@ def run():
         group = None
         for (opt, arg) in optlist:
             if opt == '-n': hostname = arg
+            elif opt == '-v':
+                print '%s (python %s)'%(roundup_version, sys.version.split()[0])
+                return
             elif opt == '-p': port = int(arg)
             elif opt == '-u': user = arg
-            elif opt == '-d': pidfile = abspath(arg)
-            elif opt == '-l': logfile = abspath(arg)
+            elif opt == '-g': group = arg
+            elif opt == '-d': pidfile = os.path.abspath(arg)
+            elif opt == '-l': logfile = os.path.abspath(arg)
             elif opt == '-h': usage()
             elif opt == '-N': RoundupRequestHandler.LOG_IPADDRESS = 0
             elif opt == '-c': svc_args = [opt] + args; args = None
@@ -445,6 +470,33 @@ def run():
 
         if pidfile and not logfile:
             raise ValueError, _("logfile *must* be specified if pidfile is")
+  
+        # obtain server before changing user id - allows to use port <
+        # 1024 if started as root
+        address = (hostname, port)
+
+        try:
+            httpd = server_class(address, RoundupRequestHandler)
+        except socket.error, e:
+            if e[0] == errno.EADDRINUSE:
+                raise socket.error, \
+                      _("Unable to bind to port %s, port already in use." % port)
+            raise
+
+        if group is not None and hasattr(os, 'getuid'):
+            # if root, setgid to the running user
+            if not os.getuid():
+                try:
+                    import grp
+                except ImportError:
+                    raise ValueError, _("Can't change groups - no grp module")
+                try:
+                    gid = grp.getgrnam(group)[2]
+                except KeyError:
+                    raise ValueError,_("Group %(group)s doesn't exist")%locals()
+                os.setgid(gid)
+            elif os.getuid():
+                print _('WARNING: ignoring "-g" argument, not root')
 
         if hasattr(os, 'getuid'):
             # if root, setuid to the running user
@@ -473,19 +525,19 @@ def run():
                     name, home = arg.split('=')
                 except ValueError:
                     raise ValueError, _("Instances must be name=home")
-                d[name] = home
+                d[name] = os.path.abspath(home)
             RoundupRequestHandler.TRACKER_HOMES = d
     except SystemExit:
         raise
+    except ValueError:
+        usage(error())
     except:
-        exc_type, exc_value = sys.exc_info()[:2]
-        usage('%s: %s'%(exc_type, exc_value))
+        print error()
+        sys.exit(1)
 
     # we don't want the cgi module interpreting the command-line args ;)
     sys.argv = sys.argv[:1]
-    address = (hostname, port)
 
-    # fork?
     if pidfile:
         if not hasattr(os, 'fork'):
             print "Sorry, you can't run the server as a daemon on this" \
@@ -503,8 +555,11 @@ def run():
         # appending, unbuffered
         sys.stdout = sys.stderr = open(logfile, 'a', 0)
 
-    httpd = BaseHTTPServer.HTTPServer(address, RoundupRequestHandler)
-    print _('Roundup server started on %(address)s')%locals()
+    if success_message:
+        print success_message
+    else:
+        print _('Roundup server started on %(address)s')%locals()
+
     try:
         httpd.serve_forever()
     except KeyboardInterrupt:
