@@ -15,7 +15,8 @@
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 # 
-# $Id$ 
+# $Id$
+from __future__ import nested_scopes
 
 import unittest, time
 
@@ -121,6 +122,8 @@ class DateTestCase(unittest.TestCase):
 
     def testOffsetSub(self):
         ae = self.assertEqual
+        date = Date('2000-12-01') - Interval('- 1d')
+
         date = Date('2000-01-01') - Interval('- 2y 2m')
         ae(str(date), '2002-03-01.00:00:00')
         date = Date('2000-01-01') - Interval('2m')
@@ -262,14 +265,14 @@ class DateTestCase(unittest.TestCase):
         ae(str(Interval('1y') + Interval('1y')), '+ 2y')
         ae(str(Interval('1y') + Interval('1m')), '+ 1y 1m')
         ae(str(Interval('1y') + Interval('2:40')), '+ 1y 2:40')
-        ae(str(Interval('1y') + Interval('- 1y')), '')
-        ae(str(Interval('- 1y') + Interval('1y')), '')
+        ae(str(Interval('1y') + Interval('- 1y')), '00:00')
+        ae(str(Interval('- 1y') + Interval('1y')), '00:00')
         ae(str(Interval('- 1y') + Interval('- 1y')), '- 2y')
         ae(str(Interval('1y') + Interval('- 1m')), '+ 11m')
         ae(str(Interval('1:00') + Interval('1:00')), '+ 2:00')
         ae(str(Interval('0:50') + Interval('0:50')), '+ 1:40')
-        ae(str(Interval('1:50') + Interval('- 1:50')), '')
-        ae(str(Interval('- 1:50') + Interval('1:50')), '')
+        ae(str(Interval('1:50') + Interval('- 1:50')), '00:00')
+        ae(str(Interval('- 1:50') + Interval('1:50')), '00:00')
         ae(str(Interval('- 1:50') + Interval('- 1:50')), '- 3:40')
         ae(str(Interval('1:59:59') + Interval('00:00:01')), '+ 2:00')
         ae(str(Interval('2:00') + Interval('- 00:00:01')), '+ 1:59:59')
@@ -279,11 +282,11 @@ class DateTestCase(unittest.TestCase):
         ae(str(Interval('1y') - Interval('- 1y')), '+ 2y')
         ae(str(Interval('1y') - Interval('- 1m')), '+ 1y 1m')
         ae(str(Interval('1y') - Interval('- 2:40')), '+ 1y 2:40')
-        ae(str(Interval('1y') - Interval('1y')), '')
+        ae(str(Interval('1y') - Interval('1y')), '00:00')
         ae(str(Interval('1y') - Interval('1m')), '+ 11m')
         ae(str(Interval('1:00') - Interval('- 1:00')), '+ 2:00')
         ae(str(Interval('0:50') - Interval('- 0:50')), '+ 1:40')
-        ae(str(Interval('1:50') - Interval('1:50')), '')
+        ae(str(Interval('1:50') - Interval('1:50')), '00:00')
         ae(str(Interval('1:59:59') - Interval('- 00:00:01')), '+ 2:00')
         ae(str(Interval('2:00') - Interval('00:00:01')), '+ 1:59:59')
 
@@ -335,9 +338,47 @@ class DateTestCase(unittest.TestCase):
         ae(str(Date('2003-5', add_granularity=1)), '2003-05-31.23:59:59')
         ae(str(Interval('+1w', add_granularity=1)), '+ 14d')
         ae(str(Interval('-2m 3w', add_granularity=1)), '- 2m 14d')
+        
+    def testIntervalPretty(self):
+        def ae(spec, pretty):
+            self.assertEqual(Interval(spec).pretty(), pretty)
+        ae('2y', 'in 2 years')
+        ae('1y', 'in 1 year')
+        ae('2m', 'in 2 months')
+        ae('1m 30d', 'in 2 months')
+        ae('60d', 'in 2 months')
+        ae('59d', 'in 1 month')
+        ae('1m', 'in 1 month')
+        ae('29d', 'in 1 month')
+        ae('28d', 'in 4 weeks')
+        ae('8d', 'in 1 week')
+        ae('7d', 'in 7 days')
+        ae('1w', 'in 7 days')
+        ae('2d', 'in 2 days')
+        ae('1d', 'tomorrow')
+        ae('02:00:00', 'in 2 hours')
+        ae('01:59:00', 'in 1 3/4 hours')
+        ae('01:45:00', 'in 1 3/4 hours')
+        ae('01:30:00', 'in 1 1/2 hours')
+        ae('01:29:00', 'in 1 1/4 hours')
+        ae('01:00:00', 'in an hour')        
+        ae('00:30:00', 'in 1/2 an hour')
+        ae('00:15:00', 'in 1/4 hour')
+        ae('00:02:00', 'in 2 minutes')
+        ae('00:01:00', 'in 1 minute')
+        ae('00:00:30', 'in a moment')
+        ae('-00:00:30', 'just now')
+        ae('-1d', 'yesterday')
+        ae('-1y', '1 year ago')
+        ae('-2y', '2 years ago')
 
-def suite():
-    return unittest.makeSuite(DateTestCase, 'test')
+def test_suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(DateTestCase))
+    return suite
 
+if __name__ == '__main__':
+    runner = unittest.TextTestRunner()
+    unittest.main(testRunner=runner)
 
 # vim: set filetype=python ts=4 sw=4 et si
