@@ -10,37 +10,68 @@
 #
 # $Id$
 
-import unittest, cStringIO, tempfile, os, shutil, errno, imp, sys, difflib
-import rfc822
+# TODO: test bcc
 
-# Note: Should parse emails according to RFC2822 instead of performing a
-# literal string comparision.  Parsing the messages allows the tests to work for
-# any legal serialization of an email.
-#try :
-#    import email
-#except ImportError :
-#    import rfc822 as email
+import unittest, tempfile, os, shutil, errno, imp, sys, difflib, rfc822
 
-from roundup.mailgw import MailGW, Unauthorized, uidFromAddress
-from roundup import init, instance, rfc2822
+from cStringIO import StringIO
+
+if not os.environ.has_key('SENDMAILDEBUG'):
+    os.environ['SENDMAILDEBUG'] = 'mail-test.log'
+SENDMAILDEBUG = os.environ['SENDMAILDEBUG']
+
+from roundup.mailgw import MailGW, Unauthorized, uidFromAddress, \
+    parseContent, IgnoreLoop, IgnoreBulk
+from roundup import init, instance, rfc2822, __version__
+
+
+class Message(rfc822.Message):
+    """String-based Message class with equivalence test."""
+    def __init__(self, s):
+        rfc822.Message.__init__(self, StringIO(s.strip()))
+        
+    def __eq__(self, other):
+        return (self.dict == other.dict and
+                self.fp.read() == other.fp.read()) 
 
-# TODO: make this output only enough equal lines for context, not all of
-# them
 class DiffHelper:
+    def compareMessages(self, new, old):
+        """Compare messages for semantic equivalence."""
+        new, old = Message(new), Message(old)
+        del new['date'], old['date']
+
+        if not new == old:
+            res = []
+
+            for key in new.keys():
+                if key.lower() == 'x-roundup-version':
+                    # version changes constantly, so handle it specially
+                    if new[key] != __version__:
+                        res.append('  %s: %s != %s' % (key, __version__,
+                            new[key]))
+                elif new[key] != old[key]:
+                    res.append('  %s: %s != %s' % (key, old[key], new[key]))
+
+            body_diff = self.compareStrings(new.fp.read(), old.fp.read())
+            if body_diff:
+                res.append('')
+                res.extend(body_diff)
+
+            if res:
+                res.insert(0, 'Generated message not correct (diff follows):')
+                raise AssertionError, '\n'.join(res)
+    
     def compareStrings(self, s2, s1):
         '''Note the reversal of s2 and s1 - difflib.SequenceMatcher wants
            the first to be the "original" but in the calls in this file,
            the second arg is the original. Ho hum.
         '''
-        # we have to special-case the Date: header here 'cos we can't test
-        # for it properly
-        l1=s1.strip().split('\n')
-        l2=[x for x in s2.strip().split('\n') if not x.startswith('Date: ')]
+        l1 = s1.strip().split('\n')
+        l2 = s2.strip().split('\n')
         if l1 == l2:
             return
-
         s = difflib.SequenceMatcher(None, l1, l2)
-        res = ['Generated message not correct (diff follows):']
+        res = []
         for value, s1s, s1e, s2s, s2e in s.get_opcodes():
             if value == 'equal':
                 for i in range(s1s, s1e):
@@ -56,7 +87,7 @@ class DiffHelper:
                     res.append('- %s'%l1[i])
                     res.append('+ %s'%l2[j])
 
-        raise AssertionError, '\n'.join(res)
+        return res
 
 class MailgwTestCase(unittest.TestCase, DiffHelper):
     count = 0
@@ -72,55 +103,62 @@ class MailgwTestCase(unittest.TestCase, DiffHelper):
         init.install(self.dirname, 'templates/classic')
         init.write_select_db(self.dirname, 'anydbm')
         init.initialise(self.dirname, 'sekrit')
+
         # check we can load the package
         self.instance = instance.open(self.dirname)
+
         # and open the database
         self.db = self.instance.open('admin')
-        self.db.user.create(username='Chef', address='chef@bork.bork.bork',
-            realname='Bork, Chef', roles='User')
-        self.db.user.create(username='richard', address='richard@test',
-            roles='User')
-        self.db.user.create(username='mary', address='mary@test',
+        self.chef_id = self.db.user.create(username='Chef',
+            address='chef@bork.bork.bork', realname='Bork, Chef', roles='User')
+        self.richard_id = self.db.user.create(username='richard',
+            address='richard@test', roles='User')
+        self.mary_id = self.db.user.create(username='mary', address='mary@test',
             roles='User', realname='Contrary, Mary')
-        self.db.user.create(username='john', address='john@test',
+        self.john_id = self.db.user.create(username='john', address='john@test',
             alternate_addresses='jondoe@test\njohn.doe@test', roles='User',
             realname='John Doe')
 
     def tearDown(self):
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            os.remove(os.environ['SENDMAILDEBUG'])
+        if os.path.exists(SENDMAILDEBUG):
+            os.remove(SENDMAILDEBUG)
         self.db.close()
         try:
             shutil.rmtree(self.dirname)
         except OSError, error:
             if error.errno not in (errno.ENOENT, errno.ESRCH): raise
 
-    def handleMessage(self, message):
+    def _handle_mail(self, message):
         handler = self.instance.MailGW(self.instance, self.db)
         handler.trapExceptions = 0
-        nodeid = handler.main(message)
-        # handler will probably close & reopen the db
+        ret = handler.main(StringIO(message))
+        # handler can close the db on us and open a new one
         self.db = handler.db
-        return nodeid
+        return ret
+        
+    def _get_mail(self):
+        f = open(SENDMAILDEBUG)
+        try:
+            return f.read()
+        finally:
+            f.close()
 
     def testEmptyMessage(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        nodeid = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
 Cc: richard@test
+Reply-To: chef@bork.bork.bork
 Message-Id: <dummy_test_message_id>
 Subject: [issue] Testing...
 
 ''')
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
+        assert not os.path.exists(SENDMAILDEBUG)
         self.assertEqual(self.db.issue.get(nodeid, 'title'), 'Testing...')
 
     def doNewIssue(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        nodeid = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
@@ -130,13 +168,10 @@ Subject: [issue] Testing...
 
 This is a test submission of a new issue.
 ''')
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
+        assert not os.path.exists(SENDMAILDEBUG)
         l = self.db.issue.get(nodeid, 'nosy')
         l.sort()
-        self.assertEqual(l, ['3', '4'])
+        self.assertEqual(l, [self.chef_id, self.richard_id])
         return nodeid
 
     def testNewIssue(self):
@@ -144,7 +179,7 @@ This is a test submission of a new issue.
 
     def testNewIssueNosy(self):
         self.instance.config.ADD_AUTHOR_TO_NOSY = 'yes'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        nodeid = self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
@@ -154,34 +189,13 @@ Subject: [issue] Testing...
 
 This is a test submission of a new issue.
 ''')
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
+        assert not os.path.exists(SENDMAILDEBUG)
         l = self.db.issue.get(nodeid, 'nosy')
         l.sort()
-        self.assertEqual(l, ['3', '4'])
-
-    def testNewUser(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
-  charset="iso-8859-1"
-From: Frank Fiddle <frank@fiddle.com>
-To: issue_tracker@your.tracker.email.domain.example
-Message-Id: <dummy_test_message_id>
-Subject: [issue] Testing...
-
-This is a test submission of a new issue.
-''')
-        userlist = self.db.user.list()
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
-        self.assertNotEqual(userlist, self.db.user.list(),
-            "user not created when it should have been")
+        self.assertEqual(l, [self.chef_id, self.richard_id])
 
     def testAlternateAddress(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: John Doe <john.doe@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -190,16 +204,13 @@ Subject: [issue] Testing...
 
 This is a test submission of a new issue.
 ''')
-        userlist = self.db.user.list()
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
+        userlist = self.db.user.list()        
+        assert not os.path.exists(SENDMAILDEBUG)
         self.assertEqual(userlist, self.db.user.list(),
             "user created when it shouldn't have been")
 
     def testNewIssueNoClass(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
@@ -209,13 +220,12 @@ Subject: Testing...
 
 This is a test submission of a new issue.
 ''')
-        nodeid = self.handleMessage(message)
-        if os.path.exists(os.environ['SENDMAILDEBUG']):
-            error = open(os.environ['SENDMAILDEBUG']).read()
-            self.assertEqual('no error', error)
+        assert not os.path.exists(SENDMAILDEBUG)
 
     def testNewIssueAuthMsg(self):
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        # TODO: fix the damn config - this is apalling
+        self.db.config.MESSAGES_TO_AUTHOR = 'yes'
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: Chef <chef@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
@@ -224,11 +234,7 @@ Subject: [issue] Testing... [nosy=mary; assignedto=richard]
 
 This is a test submission of a new issue.
 ''')
-        # TODO: fix the damn config - this is apalling
-        self.db.config.MESSAGES_TO_AUTHOR = 'yes'
-        nodeid = self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, mary@test, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -271,7 +277,7 @@ _______________________________________________________________________
 
     def testSimpleFollowup(self):
         self.doNewIssue()
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: mary <mary@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -281,8 +287,7 @@ Subject: [issue1] Testing...
 
 This is a second followup
 ''')
-        self.handleMessage(message)
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -314,7 +319,7 @@ _______________________________________________________________________
     def testFollowup(self):
         self.doNewIssue()
 
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -324,12 +329,12 @@ Subject: [issue1] Testing... [assignedto=mary; nosy=+john]
 
 This is a followup
 ''')
-        self.handleMessage(message)
         l = self.db.issue.get('1', 'nosy')
         l.sort()
-        self.assertEqual(l, ['3', '4', '5', '6'])
+        self.assertEqual(l, [self.chef_id, self.richard_id, self.mary_id,
+            self.john_id])
 
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, john@test, mary@test
 Content-Type: text/plain; charset=utf-8
@@ -362,7 +367,7 @@ _______________________________________________________________________
 
     def testFollowupTitleMatch(self):
         self.doNewIssue()
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -372,9 +377,7 @@ Subject: Re: Testing... [assignedto=mary; nosy=+john]
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, john@test, mary@test
 Content-Type: text/plain; charset=utf-8
@@ -408,7 +411,7 @@ _______________________________________________________________________
     def testFollowupNosyAuthor(self):
         self.doNewIssue()
         self.db.config.ADD_AUTHOR_TO_NOSY = 'yes'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: john@test
 To: issue_tracker@your.tracker.email.domain.example
@@ -418,9 +421,8 @@ Subject: [issue1] Testing...
 
 This is a followup
 ''')
-        self.handleMessage(message)
 
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -454,7 +456,7 @@ _______________________________________________________________________
     def testFollowupNosyRecipients(self):
         self.doNewIssue()
         self.db.config.ADD_RECIPIENTS_TO_NOSY = 'yes'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard@test
 To: issue_tracker@your.tracker.email.domain.example
@@ -465,9 +467,7 @@ Subject: [issue1] Testing...
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork
 Content-Type: text/plain; charset=utf-8
@@ -502,7 +502,7 @@ _______________________________________________________________________
         self.doNewIssue()
         self.db.config.ADD_AUTHOR_TO_NOSY = 'yes'
         self.db.config.MESSAGES_TO_AUTHOR = 'yes'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: john@test
 To: issue_tracker@your.tracker.email.domain.example
@@ -512,9 +512,7 @@ Subject: [issue1] Testing...
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, john@test, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -548,7 +546,7 @@ _______________________________________________________________________
     def testFollowupNoNosyAuthor(self):
         self.doNewIssue()
         self.instance.config.ADD_AUTHOR_TO_NOSY = 'no'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: john@test
 To: issue_tracker@your.tracker.email.domain.example
@@ -558,9 +556,7 @@ Subject: [issue1] Testing...
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -593,7 +589,7 @@ _______________________________________________________________________
     def testFollowupNoNosyRecipients(self):
         self.doNewIssue()
         self.instance.config.ADD_RECIPIENTS_TO_NOSY = 'no'
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard@test
 To: issue_tracker@your.tracker.email.domain.example
@@ -604,9 +600,7 @@ Subject: [issue1] Testing...
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork
 Content-Type: text/plain; charset=utf-8
@@ -639,7 +633,7 @@ _______________________________________________________________________
     def testFollowupEmptyMessage(self):
         self.doNewIssue()
 
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -648,18 +642,18 @@ In-Reply-To: <dummy_test_message_id>
 Subject: [issue1] Testing... [assignedto=mary; nosy=+john]
 
 ''')
-        self.handleMessage(message)
         l = self.db.issue.get('1', 'nosy')
         l.sort()
-        self.assertEqual(l, ['3', '4', '5', '6'])
+        self.assertEqual(l, [self.chef_id, self.richard_id, self.mary_id,
+            self.john_id])
 
         # should be no file created (ie. no message)
-        assert not os.path.exists(os.environ['SENDMAILDEBUG'])
+        assert not os.path.exists(SENDMAILDEBUG)
 
     def testNosyRemove(self):
         self.doNewIssue()
 
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -668,13 +662,12 @@ In-Reply-To: <dummy_test_message_id>
 Subject: [issue1] Testing... [nosy=-richard]
 
 ''')
-        self.handleMessage(message)
         l = self.db.issue.get('1', 'nosy')
         l.sort()
-        self.assertEqual(l, ['3'])
+        self.assertEqual(l, [self.chef_id])
 
         # NO NOSY MESSAGE SHOULD BE SENT!
-        self.assert_(not os.path.exists(os.environ['SENDMAILDEBUG']))
+        assert not os.path.exists(SENDMAILDEBUG)
 
     def testNewUserAuthor(self):
         # first without the permission
@@ -686,7 +679,7 @@ Subject: [issue1] Testing... [nosy=-richard]
         self.db.security.hasPermission('Email Registration', anonid)
         l = self.db.user.list()
         l.sort()
-        s = '''Content-Type: text/plain;
+        message = '''Content-Type: text/plain;
   charset="iso-8859-1"
 From: fubar <fubar@bork.bork.bork>
 To: issue_tracker@your.tracker.email.domain.example
@@ -695,8 +688,7 @@ Subject: [issue] Testing...
 
 This is a test submission of a new issue.
 '''
-        message = cStringIO.StringIO(s)
-        self.assertRaises(Unauthorized, self.handleMessage, message)
+        self.assertRaises(Unauthorized, self._handle_mail, message)
         m = self.db.user.list()
         m.sort()
         self.assertEqual(l, m)
@@ -704,15 +696,14 @@ This is a test submission of a new issue.
         # now with the permission
         p = self.db.security.getPermission('Email Registration')
         self.db.security.role['anonymous'].permissions=[p]
-        message = cStringIO.StringIO(s)
-        self.handleMessage(message)
+        self._handle_mail(message)
         m = self.db.user.list()
         m.sort()
         self.assertNotEqual(l, m)
 
     def testEnc01(self):
         self.doNewIssue()
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: mary <mary@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -726,8 +717,7 @@ Content-Transfer-Encoding: quoted-printable
 A message with encoding (encoded oe =F6)
 
 ''')
-        self.handleMessage(message)
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -759,7 +749,7 @@ _______________________________________________________________________
 
     def testMultipartEnc01(self):
         self.doNewIssue()
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: mary <mary@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -780,8 +770,7 @@ Content-Transfer-Encoding: quoted-printable
 A message with first part encoded (encoded oe =F6)
 
 ''')
-        self.handleMessage(message)
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork, richard@test
 Content-Type: text/plain; charset=utf-8
@@ -812,7 +801,7 @@ _______________________________________________________________________
 
     def testContentDisposition(self):
         self.doNewIssue()
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: mary <mary@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -837,7 +826,6 @@ xxxxxx
 
 --bCsyhTFzCvuiizWE--
 ''')
-        self.handleMessage(message)
         messages = self.db.issue.get('1', 'messages')
         messages.sort()
         file = self.db.msg.get(messages[-1], 'files')[0]
@@ -846,7 +834,7 @@ xxxxxx
     def testFollowupStupidQuoting(self):
         self.doNewIssue()
 
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -856,9 +844,7 @@ Subject: Re: "[issue1] Testing... "
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
-        self.compareStrings(open(os.environ['SENDMAILDEBUG']).read(),
+        self.compareMessages(self._get_mail(),
 '''FROM: roundup-admin@your.tracker.email.domain.example
 TO: chef@bork.bork.bork
 Content-Type: text/plain; charset=utf-8
@@ -907,7 +893,7 @@ This is a followup
 
         messages = self.db.issue.get(nodeid, 'messages')
 
-        message = cStringIO.StringIO('''Content-Type: text/plain;
+        self._handle_mail('''Content-Type: text/plain;
   charset="iso-8859-1"
 From: richard <richard@test>
 To: issue_tracker@your.tracker.email.domain.example
@@ -922,15 +908,13 @@ Blah blah wrote:
 
 This is a followup
 ''')
-        self.handleMessage(message)
-
         # figure the new message id
         newmessages = self.db.issue.get(nodeid, 'messages')
         for msg in messages:
             newmessages.remove(msg)
         messageid = newmessages[0]
 
-        self.compareStrings(self.db.msg.get(messageid, 'content'), expect)
+        self.compareMessages(self.db.msg.get(messageid, 'content'), expect)
 
     def testUserLookup(self):
         i = self.db.user.create(username='user1', address='user1@foo.com')
@@ -940,6 +924,12 @@ This is a followup
         self.assertEqual(uidFromAddress(self.db, ('', 'USER2@foo.com'), 0), i)
         self.assertEqual(uidFromAddress(self.db, ('', 'user2@foo.com'), 0), i)
 
+    def testUserAlternateLookup(self):
+        i = self.db.user.create(username='user1', address='user1@foo.com',
+                                alternate_addresses='user1@bar.com')
+        self.assertEqual(uidFromAddress(self.db, ('', 'user1@bar.com'), 0), i)
+        self.assertEqual(uidFromAddress(self.db, ('', 'USER1@bar.com'), 0), i)
+
     def testUserCreate(self):
         i = uidFromAddress(self.db, ('', 'user@foo.com'), 1)
         self.assertNotEqual(uidFromAddress(self.db, ('', 'user@bar.com'), 1), i)
@@ -951,10 +941,89 @@ This is a followup
         self.assertEqual(rfc2822.encode_header(ascii_header), ascii_header)
         self.assertEqual(rfc2822.encode_header(unicode_header), unicode_encoded)
 
-def suite():
-    l = [unittest.makeSuite(MailgwTestCase),
-    ]
-    return unittest.TestSuite(l)
+    def testRegistrationConfirmation(self):
+        otk = "Aj4euk4LZSAdwePohj90SME5SpopLETL"
+        self.db.getOTKManager().set(otk, username='johannes')
+        self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Message-Id: <dummy_test_message_id>
+Subject: Re: Complete your registration to Roundup issue tracker
+ -- key %s
 
+This is a test confirmation of registration.
+''' % otk)
+        self.db.user.lookup('johannes')
+
+    def testFollowupOnNonIssue(self):
+        self.db.keyword.create(name='Foo')
+        self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: richard <richard@test>
+To: issue_tracker@your.tracker.email.domain.example
+Message-Id: <followup_dummy_id>
+In-Reply-To: <dummy_test_message_id>
+Subject: [keyword1] Testing... [name=Bar]
+
+''')        
+        self.assertEqual(self.db.keyword.get('1', 'name'), 'Bar')
+
+    def testResentFrom(self):
+        nodeid = self._handle_mail('''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+Resent-From: mary <mary@test>
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Message-Id: <dummy_test_message_id>
+Subject: [issue] Testing...
+
+This is a test submission of a new issue.
+''')
+        assert not os.path.exists(SENDMAILDEBUG)
+        l = self.db.issue.get(nodeid, 'nosy')
+        l.sort()
+        self.assertEqual(l, [self.richard_id, self.mary_id])
+        return nodeid
+
+
+    def testDejaVu(self):
+        self.assertRaises(IgnoreLoop, self._handle_mail,
+            '''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+X-Roundup-Loop: hello
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Message-Id: <dummy_test_message_id>
+Subject: Re: [issue] Testing...
+
+Hi, I've been mis-configured to loop messages back to myself.
+''')
+
+    def testItsBulkStupid(self):
+        self.assertRaises(IgnoreBulk, self._handle_mail,
+            '''Content-Type: text/plain;
+  charset="iso-8859-1"
+From: Chef <chef@bork.bork.bork>
+Precedence: bulk
+To: issue_tracker@your.tracker.email.domain.example
+Cc: richard@test
+Message-Id: <dummy_test_message_id>
+Subject: Re: [issue] Testing...
+
+Hi, I'm on holidays, and this is a dumb auto-responder.
+''')
+
+def test_suite():
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(MailgwTestCase))
+    return suite
+
+if __name__ == '__main__':
+    runner = unittest.TextTestRunner()
+    unittest.main(testRunner=runner)
 
 # vim: set filetype=python ts=4 sw=4 et si
